@@ -241,7 +241,8 @@ class FrescoApp(RouteCollection):
         return Response.method_not_allowed(valid_methods)
 
     @contextmanager
-    def requestcontext(self, url='http://localhost/', environ=None, **kwargs):
+    def requestcontext(self, url='http://localhost/', environ=None,
+                       wsgi_input=b'', **kwargs):
         """
         Return the global :class:`fresco.requestcontext.RequestContext`
         instance, populated with a new request object modelling default
@@ -266,8 +267,15 @@ class FrescoApp(RouteCollection):
         if ':' in user:
             user, _ = user.split(':')[0]
 
+        if isinstance(wsgi_input, bytes):
+            wsgi_input = BytesIO(wsgi_input)
+
         env_overrides = environ or {}
-        env_overrides.update(kwargs)
+        for key, value in kwargs.items():
+            key = key.replace('wsgi_', 'wsgi.')
+            if '.' not in key:
+                value = unicode_to_environ(value)
+            env_overrides[key] = value
 
         environ = {
             'REQUEST_METHOD': 'GET',
@@ -280,7 +288,7 @@ class FrescoApp(RouteCollection):
             'REMOTE_ADDR': '127.0.0.1',
             'wsgi.version': (1, 0),
             'wsgi.url_scheme': url.scheme or 'http',
-            'wsgi.input': BytesIO(),
+            'wsgi.input': wsgi_input,
             'wsgi.errors': sys.stderr,
             'wsgi.multithread': True,
             'wsgi.multiprocess': True,
