@@ -147,9 +147,9 @@ class StrConverter(Converter):
 
     def from_string(self, s):
         """
-        Return ``s`` converted to an ``str`` object.
+        Return ``s`` converted to a (unicode) string type
         """
-        return str(s)
+        return s
 
 
 class AnyConverter(StrConverter):
@@ -686,11 +686,16 @@ class RouteCollection(object):
         if isinstance(viewspec, string_types) and ':' in viewspec:
             viewspec, remainder = viewspec.split(':', 1)
             delegated_route = self.routefor(viewspec)
-            p1, args, kwargs = delegated_route.path(*args, **kwargs)
-            rc = delegated_route.routecollectionfactory(*args, **kwargs)
-            rc.pathfor(remainder, *args, **kwargs)
-            p2 = delegated_route.routecollectionfactory(*args, **kwargs)\
-                                .pathfor(remainder, *args, **kwargs)
+
+            p1, remainder_args, remainder_kwargs = \
+                    delegated_route.path(*args, **kwargs)
+
+            factory_args = args[:-len(remainder_args)]
+            factory_kwargs = {k: v for k, v in kwargs.items()
+                                   if k not in remainder_kwargs}
+            rc = delegated_route.routecollectionfactory(*factory_args,
+                                                        **factory_kwargs)
+            p2 = rc.pathfor(remainder, *args, **kwargs)
             return p1 + p2
 
         return self.routefor(viewspec).path(*args, **kwargs)[0]
