@@ -4,6 +4,7 @@ from nose.tools import assert_raises
 from flea import TestAgent
 from fresco import FrescoApp, GET, POST, Response
 from fresco.exceptions import BadRequest, NotFound
+from fresco.compat import ustring_type
 from fresco.core import urlfor
 from fresco.routing import Route, DelegateRoute, RouteCollection, \
                            RouteKwarg, CookieArg, SessionArg, FormArg, \
@@ -360,11 +361,6 @@ class TestDelegatedRoutes(object):
 
     def test_urlfor_with_dynamic_delegated_route_and_view_self(self):
 
-        # XXX: maintain context.routecollections[] listing nested
-        # RouteCollections associated with the request
-        #
-        # Each entry should contain (routecollection, urlbase) so we can
-        # quickly work out URLs at each level up the tree
         result = []
 
         class MyRoutes(object):
@@ -382,6 +378,22 @@ class TestDelegatedRoutes(object):
             app.view()
             assert result == ['http://localhost/two/3/view']
 
+    def test_urlgeneration_with_dynamic_routes(self):
+
+        class Routable(object):
+            __routes__ = [Route('/<b:int>', GET, 'view', name='y')]
+
+            def __init__(self, a):
+                pass
+
+            def view(self, b):
+                pass
+
+        app = FrescoApp()
+        app.delegate('/<a:str>', Routable, dynamic=True, name='x')
+        with app.requestcontext('/two/2/view'):
+            assert urlfor('x:y', a='a', b=1) == 'http://localhost/a/1'
+
     def test_delegated_routes_can_be_included(self):
 
         view = Mock()
@@ -417,3 +429,11 @@ class TestDelegatedRoutes(object):
         with outer.requestcontext('/foo'):
             outer.view()
             view.assert_called()
+
+
+class TestConverters(object):
+
+    def test_str_converter_returns_unicode(self):
+        from fresco.routing import StrConverter
+        s = ustring_type('abc')
+        assert isinstance(StrConverter().from_string(s), ustring_type)
