@@ -1,13 +1,17 @@
+# coding=UTF-8
 # Copyright (c) 2009 Oliver Cope. All rights reserved.
 # See LICENSE.txt for terms of redistribution and use.
 
-from cStringIO import StringIO
+from __future__ import unicode_literals
+from io import BytesIO
 
 from nose.tools import assert_equal
 from flea import TestAgent
 
+from fresco.compat import quote
 from fresco.util.http import parse_querystring, parse_post
-from fresco import FrescoApp, Response, GET, POST
+from fresco.util.wsgi import unicode_to_environ
+from fresco import FrescoApp, context, Response, POST
 
 from . import form_data
 
@@ -35,12 +39,15 @@ class TestParseQueryString(object):
     def test_escaped_chars(self):
         assert_equal(self.p('%20==c%3D'), [(' ', '=c=')])
 
+    def test_charset(self):
+        assert_equal(self.p('a=el%20ni%C3%B1o'), [('a', 'el niño')])
+
 
 class TestParseMultipart(object):
 
     def test_multipart(self):
         for data in form_data.multipart_samples:
-            io = StringIO(data['data'])
+            io = BytesIO(data['data'])
             io.seek(0)
             environ = {
                 'CONTENT_LENGTH': data['content_length'],
@@ -57,7 +64,7 @@ class TestParseMultipart(object):
             assert_equal(parsed[0], ("empty-text-input", ""))
             assert_equal(parsed[2], ("text-input-ascii", "abcdef"))
             assert_equal(parsed[3], ("text-input-unicode",
-                                    "\xce\xb1\xce\xb2\xce\xb3\xce\xb4"\
+                                    b"\xce\xb1\xce\xb2\xce\xb3\xce\xb4"\
                                     .decode("utf8")))
 
             fieldname, fileupload = parsed[1]
@@ -73,7 +80,8 @@ class TestParseMultipart(object):
         if the total data size exceeds request.MAX_SIZE bytes
         """
 
-        def view(request):
+        def view():
+            request = context.request
             request.MAX_MULTIPART_SIZE = 500
             request.get('f1')
             return Response(['ok'])
@@ -95,7 +103,8 @@ class TestParseMultipart(object):
 
     def test_fileupload_with_invalid_content_length(self):
 
-        def view(request):
+        def view():
+            request = context.request
             request.MAX_MULTIPART_SIZE = 500
             request.get('f1')
             return Response(['ok'])
@@ -104,7 +113,7 @@ class TestParseMultipart(object):
 
         r = TestAgent(app).post_multipart(
             files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)],
-            CONTENT_LENGTH="499",
+            CONTENT_LENGTH=unicode_to_environ("499"),
             check_status=False
         )
         assert_equal(r.response.status, "400 Bad Request")
@@ -114,7 +123,8 @@ class TestParseMultipart(object):
         Verify that multipart/form-data encoded POST data raises an exception
         if it contains a single field exceeding request.MAX_SIZE bytes
         """
-        def view(request):
+        def view():
+            request = context.request
             request.MAX_MULTIPART_SIZE = 500
             request.MAX_SIZE = 100
             request.get('f1')
@@ -123,18 +133,25 @@ class TestParseMultipart(object):
         app.route('/', POST, view)
 
         r = TestAgent(app).post_multipart(data=[('f1', 'x' * 200)],
-                                        check_status=False)
+                                          check_status=False)
         assert_equal(r.response.status, "413 Request Entity Too Large")
 
 
 class TestParseFormEncodedData(object):
 
+    char_latin1 = b'\xa3'
+    char_utf8 = b'\xc2\xa3'
+    char = char_latin1.decode('latin1')
+
+    assert char_latin1.decode('latin1') == char_utf8.decode('utf8') == char
+
     def test_formencoded_data_too_big(self):
         """
         Verify that application/x-www-form-urlencoded POST data raises an
         exception if it exceeds request.MAX_SIZE bytes
         """
-        def view(request):
+        def view():
+            request = context.request
             request.MAX_SIZE = 100
             request.get('f1')
             return Response(['ok'])
@@ -146,26 +163,22 @@ class TestParseFormEncodedData(object):
 
     def test_non_utf8_data_posted(self):
 
-        def view(request):
-            request.charset = 'latin1'
-            x = request.form['char']
-            return Response([x.encode('utf8')])
-        app = FrescoApp()
-        app.route('/', POST, view)
+        data = b'char=' + quote(self.char_latin1).encode('ascii')
+        env = {'REQUEST_METHOD': 'POST',
+               'CONTENT_LENGTH': str(len(data)),
+               'wsgi.input': BytesIO(data)}
 
-        r = TestAgent(app).post(data={'char': u'\u00a3'.encode('latin1')},
-                                check_status=False)
-        assert_equal(r.response.body.decode('utf8'), u"\u00a3")
+        with FrescoApp().requestcontext(environ=env) as c:
+            request = c.request
+            request.charset = 'latin1'
+            assert_equal(request.form['char'], self.char)
 
     def test_non_utf8_data_getted(self):
 
-        def view(request):
-            request.charset = 'latin1'
-            x = request.form['char']
-            return Response([x.encode('utf8')])
-        app = FrescoApp()
-        app.route('/', GET, view)
+        data = 'char=' + quote(self.char_latin1)
+        env = {'REQUEST_METHOD': 'GET', 'QUERY_STRING': data}
 
-        r = TestAgent(app).get(data={'char': u'\u00a3'.encode('latin1')},
-                            check_status=False)
-        assert_equal(r.response.body.decode('utf8'), u"\u00a3")
+        with FrescoApp().requestcontext(environ=env) as c:
+            request = c.request
+            request.charset = 'latin1'
+            assert_equal(request.query['char'], self.char)
